import TokenManager from 'fc-token-manager';
import { MyGlobal as app } from './globals';
import { ArgsForTokenManager, ValidObject, AxiosMiddleware } from './types';
import axios from 'axios';

const createShopifyTokenManager = (storeName: string) => {
    let tokenArgs: ArgsForTokenManager = {
        storeName: storeName,
        // REDIS_HOST: app.tokenManagerParams?.REDIS_HOST,
        // REDIS_PORT: app.tokenManagerParams?.REDIS_PORT,
        // REDIS_DB: app.tokenManagerParams?.REDIS_DB,
        // REDIS_PASS: app.tokenManagerParams?.REDIS_PASS,
        redisInstance: app.redisInstance,
        logger: app.logger,
        FlclError: app.FlclError
    };
    app.shopifyTokenManager[storeName] = TokenManager.create('shopify', tokenArgs);
    app.logger?.info({npm:'fc-shopify-middleware', className: 'axiosMiddleware', methodName: 'createShopifyTokenManager', propertyName: 'tokenArgs', propertyValue: {storeName: tokenArgs.storeName, redisInstance: tokenArgs.redisInstance?"YES":"NULL"}});
    return app.shopifyTokenManager[storeName];
}

const getShopifyTokenManager = (storeName: string) => {
    if(app.shopifyTokenManager && app.shopifyTokenManager[storeName])
        return app.shopifyTokenManager[storeName];
    else
        return createShopifyTokenManager(storeName);
}

let axiosMiddleware: AxiosMiddleware = {
    get: async (urlPath: string, configs?: ValidObject, options?: ValidObject) => {
        if(app.tokenizedCall) {
            let shopifyTokenManager = getShopifyTokenManager(options?.storeName);
            let uniqString = (Date.now() + Math.random()).toString(36).replace('.', '');
            let token = await shopifyTokenManager?.getToken(uniqString);
            if(token) {
                app.logger?.info({npm:'fc-shopify-middleware', className: 'axiosMiddleware', methodName:'get',propertyName: 'GET', propertyValue: {urlPath, configs, token}, msg: 'Making Api call'});
                return axios.get(urlPath, configs);
            } else {
                app.logger?.info({npm:'fc-shopify-middleware', className: 'axiosMiddleware', methodName:'get',propertyName: 'GET', propertyValue: {urlPath, configs, token, msg: 'Not making API call, Returning Null'}});
                return null;
            }
        } else {
            app.logger?.info({npm:'fc-shopify-middleware', className: 'axiosMiddleware', methodName:'get',propertyName: 'GET', propertyValue: {urlPath, configs}, msg: 'Normal API call'});
            return axios.get(urlPath, configs);
        }
    },
    post: async (urlPath: string, data?: ValidObject, configs?: ValidObject, options?: ValidObject) => {
        if(app.tokenizedCall) {
            let shopifyTokenManager = getShopifyTokenManager(options?.storeName);
            let uniqString = (Date.now() + Math.random()).toString(36).replace('.', '');
            let token = await shopifyTokenManager?.getToken(uniqString);
            if(token) {
                app.logger?.info({npm:'fc-shopify-middleware', className: 'axiosMiddleware', methodName:'post',propertyName: 'POST', propertyValue: {urlPath, configs, token}, msg: 'Making Api call'});
                return axios.post(urlPath, data, configs);
            } else {
                app.logger?.info({npm:'fc-shopify-middleware', className: 'axiosMiddleware', methodName:'post',propertyName: 'POST', propertyValue: {urlPath, configs, token, msg: 'Not making API call, Returning Null'}});
                return null;
            }
        } else {
            app.logger?.info({npm:'fc-shopify-middleware', className: 'axiosMiddleware', methodName:'post',propertyName: 'POST', propertyValue: {urlPath, configs}, msg: 'Normal API call'});
            return axios.post(urlPath, data, configs);
        }
    },
    put: async (urlPath: string, data?: ValidObject, configs?: ValidObject, options?: ValidObject) => {
        if (app.tokenizedCall) {
            let shopifyTokenManager = getShopifyTokenManager(options?.storeName);
            let uniqString = (Date.now() + Math.random()).toString(36).replace('.', '');
            let token = await shopifyTokenManager?.getToken(uniqString);
            if (token) {
                app.logger?.info({ npm: 'fc-shopify-middleware', className: 'axiosMiddleware', methodName: 'put', propertyName: 'put', propertyValue: { urlPath, configs, token }, msg: 'Making Api call' });
                return axios.put(urlPath, data, configs);
            } else {
                app.logger?.info({ npm: 'fc-shopify-middleware', className: 'axiosMiddleware', methodName: 'put', propertyName: 'put', propertyValue: { urlPath, configs, token, msg: 'Not making API call, Returning Null' } });
                return null;
            }
        } else {
            app.logger?.info({ npm: 'fc-shopify-middleware', className: 'axiosMiddleware', methodName: 'put', propertyName: 'put', propertyValue: { urlPath, configs }, msg: 'Normal API call' });
            return axios.put(urlPath, data, configs);
        }
    }
}

export default axiosMiddleware;
