import redis, { RedisClient } from "redis";
import Redlock from 'redlock';
import { MyGlobal as app } from './globals';

export default class Redis {
    client: RedisClient | undefined;
    redlock: Redlock | undefined;
    constructor() {
        console.log(`REDIS_HOST:PORT ${app.tokenManagerParams!.REDIS_HOST}:${app.tokenManagerParams!.REDIS_PORT} Pass: ${app.tokenManagerParams!.REDIS_PASS} DB: ${app.tokenManagerParams!.REDIS_DB}`);
        try {
            this.client = redis.createClient({
                host: app.tokenManagerParams!.REDIS_HOST,
                port: parseInt(app.tokenManagerParams!.REDIS_PORT),
                tls: true
            });
            /*this.client.auth(app.tokenManagerParams!.REDIS_PASS, () => {
                console.log('redis connected with auth ');
            });*/
            this.client.select(app.tokenManagerParams!.REDIS_DB, () => {
                console.log('redis database ' + app.tokenManagerParams!.REDIS_DB + ' connected with host ' + app.tokenManagerParams!.REDIS_HOST + ' and port ' + app.tokenManagerParams!.REDIS_PORT);
            });
            this.client.on('connect', () => {
                console.log('redis connected with host ' + app.tokenManagerParams!.REDIS_HOST + ' and port ' + app.tokenManagerParams!.REDIS_PORT);
            });
            this.redlock = new Redlock([this.client], {
                // the expected clock drift; for more details
                // see http://redis.io/topics/distlock
                driftFactor: 0.01, // time in ms
                // the max number of times Redlock will attempt
                // to lock a resource before erroring
                retryCount: 10,
                // the time in ms between attempts
                retryDelay: 200, // time in ms
                // the max time in ms randomly added to retries
                // to improve performance under high contention
                // see https://www.awsarchitectureblog.com/2015/03/backoff.html
                retryJitter: 200 // time in ms
            });
        } catch (e) {
            app.logger?.error(app.FlclError!.create({ className: 'Redis', methodName: 'constructor', cause: e, message: "Exception in connecting to redis" }));
        }
    }
}
